#include <libavr/device.h>

/* This file is a C implmentation of enough of crt0*.o to allow whole program
 * optimisation.  You can use it as follows:
 *
 *     avr-gcc -mmcu=atmega??? -Ilibavr --combine -fwhole-program -S -o a.S \
 *             a.c b.c libavr/crt/crt-hack.c
 *     avr-gcc -mmcu=atmega??? -Ilibavr -nostartfiles -o a.elf \
 *             a.S libavr/crt/crt-weak.S
 */

#if (VECTOR_SIZE == 2)
#define JMP "rjmp"
#else
#define JMP "jmp"
#endif

#define VECTOR(x) \
    void __vector_##x(void); \
    asm (".ifle (%0)\n\t" \
         ".org %1\n\t" \
         JMP " %2\n\t" \
         ".endif" \
         : : "i" (x - VECTOR_N), "i" (x * VECTOR_SIZE), "i" (__vector_##x));

__attribute__((used, naked, section(".vectors"))) void __vectors(void) {
    /* define the interrupt vector table */
    asm ("#define gs(x) x" : :); // evil hack
    VECTOR( 0)
#ifndef NOINTERRUPTS
               VECTOR( 1) VECTOR( 2) VECTOR( 3) VECTOR( 4) VECTOR( 5) \
    VECTOR( 6) VECTOR( 7) VECTOR( 8) VECTOR( 9) VECTOR(10) VECTOR(11) \
    VECTOR(12) VECTOR(13) VECTOR(14) VECTOR(15) VECTOR(16) VECTOR(17) \
    VECTOR(18) VECTOR(19) VECTOR(20) VECTOR(21) VECTOR(22) VECTOR(23) \
    VECTOR(24) VECTOR(25) VECTOR(26) VECTOR(27) VECTOR(28) VECTOR(29) \
    VECTOR(30) VECTOR(31) VECTOR(32) VECTOR(33) VECTOR(34) VECTOR(35) \
    VECTOR(36) VECTOR(37) VECTOR(38) VECTOR(39) VECTOR(40) VECTOR(41) \
    VECTOR(42) VECTOR(43) VECTOR(44) VECTOR(45) VECTOR(46) VECTOR(47) \
    VECTOR(48) VECTOR(49) VECTOR(50) VECTOR(51) VECTOR(52) VECTOR(53) \
    VECTOR(54) VECTOR(55) VECTOR(56) VECTOR(57) VECTOR(58) VECTOR(59) \
    VECTOR(60) VECTOR(61) VECTOR(62) VECTOR(63) VECTOR(64)
#endif
    asm ("#undef gs" : :);
}

__attribute__((used, naked, section(".init2"))) void __vector_0(void) {
    /* clear the __zero_reg__ and flags, setup Yreg and stack pointer */
    asm ("clr __zero_reg__\n\t"
         "ldi r28, lo8(%0)\n\t"
         "ldi r29, hi8(%0)\n\t"
         "out __SP_L__, r28\n\t"
         "out __SP_H__, r29\n\t"
         "out __SREG__, __zero_reg__"
         : : "i" (RAMEND));
}

__attribute__((used, noreturn, section(".init9"))) void main(void);
