#ifndef __LIBAVR_BITS__
#define __LIBAVR_BITS__

/* for avr-libc */
#undef bit_is_set
#undef bit_is_clear

#ifdef __GNUC__
#define sbi(var, bit) var |= 1<<bit
#define cbi(var, bit) var &= ~(1<<bit)

#else

/*
 * sbi and cbi only work on __IO_MEM()
 * for example: sbi(PORTA, 1) will work
 */
#define sbi(io, bit) \
    asm volatile ( "sbi %0, %1" : : "i" (&io - 0x20), "i" (bit) )
#define cbi(io, bit) \
    asm volatile ( "cbi %0, %1" : : "i" (&io - 0x20), "i" (bit) )
#endif

#define tbi(var, bit) var ^= 1<<bit

#define bit_is_set(io, bit) (io >> bit & 1)
#define bit_is_clear(io, bit) !bit_is_set(io, bit)

/* SBI/CBI/TBI/BIT_IS_SET/BIT_IS_CLEAR are helper functions */
#define SBI(A, B) sbi(CONCAT(A, PORT_ ## B), BIT_ ## B)
#define CBI(A, B) cbi(CONCAT(A, PORT_ ## B), BIT_ ## B)
#define TBI(A, B) tbi(CONCAT(A, PORT_ ## B), BIT_ ## B)

#define BIT_IS_SET(A, B) bit_is_set(CONCAT(A, PORT_ ## B), BIT_ ## B)
#define BIT_IS_CLEAR(A, B) bit_is_clear(CONCAT(A, PORT_ ## B), BIT_ ## B)

#define CONCAT(A, B) CONCAT_(A, B)
#define CONCAT_(A, B) A ## B

#endif
