/* AVR MEGA128 */

#define SPM_PAGESIZE    256
#define RAMEND          0x10FF
#define EEPROMEND       0x0FFF
#define FLASHEND        0x1FFFF
#define VECTOR_N        34
#define VECTOR_SIZE     4

/* This file is in memory order - to aid checking with the register summary
 * on page 362--364 of the datasheet (revision 2467R-06/08)
 *
 * For registers with named bits, the definitions of those bits follow the
 * register defintion
 */

/* io - page 88-89 */
#define PINF    __IO_MEM(0x20)
#define PINE    __IO_MEM(0x21)
#define DDRE    __IO_MEM(0x22)
#define PORTE   __IO_MEM(0x23)

/* ADC - page 242-245 */
#define ADCL    __IO_MEM(0x24)
#define ADCH    __IO_MEM(0x25)
#define ADCSRA  __IO_MEM(0x26)
#define ADPS0   0
#define ADPS1   1
#define ADPS2   2
#define ADIE    3
#define ADIF    4
#define ADFR    5
#define ADSC    6
#define ADEN    7
#define ADMUX   __IO_MEM(0x27)
#define MUX0    0
#define MUX1    1
#define MUX2    2
#define MUX3    3
#define MUX4    4
#define ADLAR   5
#define REFS0   6
#define REFS1   7

/* analog comparator - page 227 */
#define ACSR    __IO_MEM(0x28)
#define ACIS0   0
#define ACIS1   1
#define ACIC    2
#define ACIE    3
#define ACI     4
#define ACO     5
#define ACBG    6
#define ACD     7

/* USART0 - page 189-192 */
#define UBRR0L  __IO_MEM(0x29)
#define UCSR0B  __IO_MEM(0x2A)
#define TXB80   0
#define RXB80   1
#define UCSZ02  2
#define TXEN0   3
#define RXEN0   4
#define UDRIE0  5
#define TXCIE0  6
#define RXCIE0  7
#define UCSR0A  __IO_MEM(0x2B)
#define MPCM0   0
#define U2X0    1
#define UPE0    2
#define DOR0    3
#define FE0     4
#define UDRE0   5
#define TXC0    6
#define RXC0    7
#define UDR0    __IO_MEM(0x2C)

/* SPI - page 167-169 */
#define SPCR    __IO_MEM(0x2D)
#define SPR0    0
#define SPR1    1
#define CPHA    2
#define CPOL    3
#define MSTR    4
#define DORD    5
#define SPE     6
#define SPIE    7
#define SPSR    __IO_MEM(0x2E)
#define SPI2X   0
#define WCOL    6
#define SPIF    7
#define SPDR    __IO_MEM(0x2F)

/* io - page 87-88 */
#define PIND    __IO_MEM(0x30)
#define DDRD    __IO_MEM(0x31)
#define PORTD   __IO_MEM(0x32)
#define PINC    __IO_MEM(0x33)
#define DDRC    __IO_MEM(0x34)
#define PORTC   __IO_MEM(0x35)
#define PINB    __IO_MEM(0x36)
#define DDRB    __IO_MEM(0x37)
#define PORTB   __IO_MEM(0x38)
#define PINA    __IO_MEM(0x39)
#define DDRA    __IO_MEM(0x3A)
#define PORTA   __IO_MEM(0x3B)

/* eeprom - page 24 */
#define EECR    __IO_MEM(0x3C)
#define EERE    0
#define EEWE    1
#define EEMWE   2
#define EERIE   3
#define EEDR    __IO_MEM(0x3D)
#define EEARL   __IO_MEM(0x3E)
#define EEARH   __IO_MEM(0x3F)

/* special function io - page 37, 110, 145, 227 */
#define SFIOR   __IO_MEM(0x40)
#define PSR321  0
#define PSR0    1
#define PUD     2
#define ACME    3
#define TSM     7

/* watchdog - page 56 */
#define WDTCR   __IO_MEM(0x41)
#define WDP0    0
#define WDP1    1
#define WDP2    2
#define WDE     3
#define WDCE    4

/* on-chip debug - page 251 */
#define OCDR    __IO_MEM(0x42)
#define IDRD    7

/* timer2 - page 157-159 */
#define OCR2    __IO_MEM(0x43)
#define TCNT2   __IO_MEM(0x44)
#define TCCR2   __IO_MEM(0x45)
#define CS20    0
#define CS21    1
#define CS22    2
#define WGM21   3
#define COM20   4
#define COM21   5
#define WGM20   6
#define FOC2    7

/* timer1 - page 133-139 */
#define ICR1L   __IO_MEM(0x46)
#define ICR1H   __IO_MEM(0x47)
#define OCR1BL  __IO_MEM(0x48)
#define OCR1BH  __IO_MEM(0x49)
#define OCR1AL  __IO_MEM(0x4A)
#define OCR1AH  __IO_MEM(0x4B)
#define TCNT1L  __IO_MEM(0x4C)
#define TCNT1H  __IO_MEM(0x4D)
#define TCCR1B  __IO_MEM(0x4E)
#define CS10    0
#define CS11    1
#define CS12    2
#define WGM12   3
#define WGM13   4
#define ICES1   6
#define ICNC1   7
#define TCCR1A  __IO_MEM(0x4F)
#define WGM10   0
#define WGM11   1
#define COM1C0  2
#define COM1C1  3
#define COM1B0  4
#define COM1B1  5
#define COM1A0  6
#define COM1A1  7

/* timer0 - page 104-107 */
#define ASSR    __IO_MEM(0x50)
#define TCR0UB  0
#define OCR0UB  1
#define TCN0UB  2
#define AS0     3
#define OCR0    __IO_MEM(0x51)
#define TCNT0   __IO_MEM(0x52)
#define TCCR0   __IO_MEM(0x53)
#define CS00    0
#define CS01    1
#define CS02    2
#define WGM01   3
#define COM00   4
#define COM01   5
#define WGM00   6
#define FOC0    7

/* MCU control / status - page 31, 45, 54, 64, 254 */
#define MCUCSR  __IO_MEM(0x54)
#define PORF    0
#define EXTRF   1
#define BORF    2
#define WDRF    3
#define JTRF    4
#define JTD     7
#define MCUCR   __IO_MEM(0x55)
#define IVCE    0
#define IVSEL   1
#define SM2     2
#define SM0     3
#define SM1     4
#define SE      5
#define SRW10   6
#define SRE     7

/* timers - page 109, 139-141, 159-160 */
#define TIFR    __IO_MEM(0x56)
#define TOV0    0
#define OCF0    1
#define TOV1    2
#define OCF1B   3
#define OCF1A   4
#define ICF1    5
#define TOV2    6
#define OCF2    7
#define TIMSK   __IO_MEM(0x57)
#define TOIE0   0
#define OCIE0   1
#define TOIE1   2
#define OCIE1B  3
#define OCIE1A  4
#define TICIE1  5
#define TOIE2   6
#define OCIE2   7

/* external interrupts - page 91-92 */
#define EIFR    __IO_MEM(0x58)
#define INTF0   0
#define INTF1   1
#define INTF2   2
#define INTF3   3
#define INTF4   4
#define INTF5   5
#define INTF6   6
#define INTF7   7
#define EIMSK   __IO_MEM(0x59)
#define INT0    0
#define INT1    1
#define INT2    2
#define INT3    3
#define INT4    4
#define INT5    5
#define INT6    6
#define INT7    7
#define EICRB   __IO_MEM(0x5A)
#define ISC40   0
#define ISC41   1
#define ISC50   2
#define ISC51   3
#define ISC60   4
#define ISC61   5
#define ISC70   6
#define ISC71   7

/* RAM page Z select - page 14 */
#define RAMPZ   __IO_MEM(0x5B)
#define RAMPZ0  0

/* XTAL divide control - page 37 */
#define XDIV    __IO_MEM(0x5C)
#define XDIV0   0
#define XDIV1   1
#define XDIV2   2
#define XDIV3   3
#define XDIV4   4
#define XDIV5   5
#define XDIV6   6
#define XDIVEN  7

/* stack pointer / status - page 11-14 */
#define SPL     __IO_MEM(0x5D)
#define SPH     __IO_MEM(0x5E)
#define SREG    __IO_MEM(0x5F)

/* io - page 88-89 */
#define DDRF    __IO_MEM(0x61)
#define PORTF   __IO_MEM(0x62)
#define PING    __IO_MEM(0x63)
#define DDRG    __IO_MEM(0x64)
#define PORTG   __IO_MEM(0x65)

/* store program memory control / status - page 277 */
#define SPMCSR  __IO_MEM(0x68)
#define SPMEN   0
#define PGERS   1
#define PGWRT   2
#define BLBSET  3
#define RWWSRE  4
#define RWWSB   6
#define SPMIE   7

/* external interrupts - page 90 */
#define EICRA   __IO_MEM(0x6A)
#define ISC00   0
#define ISC01   1
#define ISC10   2
#define ISC11   3
#define ISC20   4
#define ISC21   5
#define ISC30   6
#define ISC31   7

/* external memory - page 31-33 */
#define XMCRB   __IO_MEM(0x6C)
#define XMM0    0
#define XMM1    1
#define XMM2    2
#define XMBK    7
#define XMCRA   __IO_MEM(0x6D)
#define SRW11   1
#define SRW00   2
#define SRW01   3
#define SRL0    4
#define SRL1    5
#define SRL2    6

/* oscillator calibration - page 42 */
#define OSCCAL  __IO_MEM(0x6F)

/* TWI - page 206-208 */
#define TWBR    __IO_MEM(0x70)
#define TWSR    __IO_MEM(0x71)
#define TWPS0   0
#define TWPS1   1
#define TWS3    3
#define TWS4    4
#define TWS5    5
#define TWS6    6
#define TWS7    7
#define TWAR    __IO_MEM(0x72)
#define TWGCE   0
#define TWA0    1
#define TWA1    2
#define TWA2    3
#define TWA3    4
#define TWA4    5
#define TWA5    6
#define TWA6    7
#define TWDR    __IO_MEM(0x73)
#define TWCR    __IO_MEM(0x74)
#define TWIE    0
#define TWEN    2
#define TWWC    3
#define TWSTO   4
#define TWSTA   5
#define TWEA    6
#define TWINT   7

/* timer1 - page 137-138 */
#define OCR1CL  __IO_MEM(0x78)
#define OCR1CH  __IO_MEM(0x79)
#define TCCR1C  __IO_MEM(0x7A)
#define FOC1C   5
#define FOC1B   6
#define FOC1A   7

/* timers - page 140-141 */
#define ETIFR   __IO_MEM(0x7C)
#define OCF1C   0
#define OCF3C   1
#define TOV3    2
#define OCF3B   3
#define OCF3A   4
#define ICF3    5
#define ETIMSK  __IO_MEM(0x7D)
#define OCIE1C  0
#define OCIE3C  1
#define TOIE3   2
#define OCIE3B  3
#define OCIE3A  4
#define TICIE3  5

/* timer3 - page 137-139 */
#define ICR3L   __IO_MEM(0x80)
#define ICR3H   __IO_MEM(0x81)
#define OCR3CL  __IO_MEM(0x82)
#define OCR3CH  __IO_MEM(0x83)
#define OCR3BL  __IO_MEM(0x84)
#define OCR3BH  __IO_MEM(0x85)
#define OCR3AL  __IO_MEM(0x86)
#define OCR3AH  __IO_MEM(0x87)
#define TCNT3L  __IO_MEM(0x88)
#define TCNT3H  __IO_MEM(0x89)
#define TCCR3B  __IO_MEM(0x8A)
#define CS30    0
#define CS31    1
#define CS32    2
#define WGM32   3
#define WGM33   4
#define ICES3   6
#define ICNC3   7
#define TCCR3A  __IO_MEM(0x8B)
#define WGM30   0
#define WGM31   1
#define COM3C0  2
#define COM3C1  3
#define COM3B0  4
#define COM3B1  5
#define COM3A0  6
#define COM3A1  7
#define TCCR3C  __IO_MEM(0x8C)
#define FOC3C   5
#define FOC3B   6
#define FOC3A   7

/* USART0 - page 189-192 */
#define UBRR0H  __IO_MEM(0x90)
#define UCSR0C  __IO_MEM(0x95)
#define UCPOL0  0
#define UCSZ00  1
#define UCSZ01  2
#define USBS0   3
#define UPM00   4
#define UPM01   5
#define UMSEL0  6

/* USART1 - page 189-192 */
#define UBRR1H  __IO_MEM(0x98)
#define UBRR1L  __IO_MEM(0x99)
#define UCSR1B  __IO_MEM(0x9A)
#define TXB81   0
#define RXB81   1
#define UCSZ12  2
#define TXEN1   3
#define RXEN1   4
#define UDRIE1  5
#define TXCIE1  6
#define RXCIE1  7
#define UCSR1A  __IO_MEM(0x9B)
#define MPCM1   0
#define U2X1    1
#define UPE1    2
#define DOR1    3
#define FE1     4
#define UDRE1   5
#define TXC1    6
#define RXC1    7
#define UDR1    __IO_MEM(0x9C)
#define UCSR1C  __IO_MEM(0x9D)
#define UCPOL1  0
#define UCSZ10  1
#define UCSZ11  2
#define USBS1   3
#define UPM10   4
#define UPM11   5
#define UMSEL1  6

/* 16bit registers */
#define ADC     __IO_MEM16(0x24)
#define EEAR    __IO_MEM16(0x3E)
#define ICR1    __IO_MEM16(0x46)
#define OCR1B   __IO_MEM16(0x48)
#define OCR1A   __IO_MEM16(0x4A)
#define TCNT1   __IO_MEM16(0x4C)
#define SP      __IO_MEM16(0x5D)
#define OCR1C   __IO_MEM16(0x78)
#define ICR3    __IO_MEM16(0x80)
#define OCR3C   __IO_MEM16(0x82)
#define OCR3B   __IO_MEM16(0x84)
#define OCR3A   __IO_MEM16(0x86)
#define TCNT3   __IO_MEM16(0x88)

/* interrupts */
#define __vector_INT0           __vector_1
#define __vector_INT1           __vector_2
#define __vector_INT2           __vector_3
#define __vector_INT3           __vector_4
#define __vector_INT4           __vector_5
#define __vector_INT5           __vector_6
#define __vector_INT6           __vector_7
#define __vector_INT7           __vector_8
#define __vector_TIMER2_COMP    __vector_9
#define __vector_TIMER2_OVF     __vector_10
#define __vector_TIMER1_CAPT    __vector_11
#define __vector_TIMER1_COMPA   __vector_12
#define __vector_TIMER1_COMPB   __vector_13
#define __vector_TIMER1_OVF     __vector_14
#define __vector_TIMER0_COMP    __vector_15
#define __vector_TIMER0_OVF     __vector_16
#define __vector_SPI_STC        __vector_17
#define __vector_USART0_RX      __vector_18
#define __vector_USART0_UDRE    __vector_19
#define __vector_USART0_TX      __vector_20
#define __vector_ADC            __vector_21
#define __vector_EE_READY       __vector_22
#define __vector_ANALOG_COMP    __vector_23
#define __vector_TIMER1_COMPC   __vector_24
#define __vector_TIMER3_CAPT    __vector_25
#define __vector_TIMER3_COMPA   __vector_26
#define __vector_TIMER3_COMPB   __vector_27
#define __vector_TIMER3_COMPC   __vector_28
#define __vector_TIMER3_OVF     __vector_29
#define __vector_USART1_RX      __vector_30
#define __vector_USART1_UDRE    __vector_31
#define __vector_USART1_TX      __vector_32
#define __vector_TWI            __vector_33
#define __vector_SPM_READY      __vector_34
