/*
 * AVR MEGA48/88/168
 * AVR MEGA48A/48PA/88A/88PA/168A/168PA/328/328P
 *
 * This file is in memory order - to aid checking with the register summary
 * on page 342-345 (rev 2545S-07/10) - also page 532-535 (rev 8271C-08/10)
 *
 * For registers with named bits, the definitions of those bits follow the
 * register defintion
 */

/* IO - page 86-87 */
#define PINB    __IO_MEM(0x23)
#define DDRB    __IO_MEM(0x24)
#define PORTB   __IO_MEM(0x25)
#define PINC    __IO_MEM(0x26)
#define DDRC    __IO_MEM(0x27)
#define PORTC   __IO_MEM(0x28)
#define PIND    __IO_MEM(0x29)
#define DDRD    __IO_MEM(0x2A)
#define PORTD   __IO_MEM(0x2B)

/* timers - page 105, 135, 157 */
#define TIFR0   __IO_MEM(0x35)
#define TOV0    0
#define OCF0A   1
#define OCF0B   2
#define TIFR1   __IO_MEM(0x36)
#define TOV1    0
#define OCF1A   1
#define OCF1B   2
#define ICF1    5
#define TIFR2   __IO_MEM(0x37)
#define TOV2    0
#define OCF2A   1
#define OCF2B   2

/* external / pin change interrupts - page 67-68 */
#define PCIFR   __IO_MEM(0x3B)
#define PCIF0   0
#define PCIF1   1
#define PCIF2   2
#define EIFR    __IO_MEM(0x3C)
#define INTF0   0
#define INTF1   1
#define EIMSK   __IO_MEM(0x3D)
#define INT0    0
#define INT1    1

/* general purpose IO - page 25 */
#define GPIOR0  __IO_MEM(0x3E)

/* eeprom - page 21 */
#define EECR    __IO_MEM(0x3F)
#define EERE    0
#define EEPE    1
#define EEMPE   2
#define EERIE   3
#define EEPM0   4
#define EEPM1   5
#define EEDR    __IO_MEM(0x40)
#define EEARL   __IO_MEM(0x41)
#define EEARH   __IO_MEM(0x42)

/* general timer / counter - page 138, 159 */
#define GTCCR   __IO_MEM(0x43)
#define PSRSYNC 0
#define PSRASY  1
#define TSM     7

/* timer 0 - page 100-104 */
#define TCCR0A  __IO_MEM(0x44)
#define WGM00   0
#define WGM01   1
#define COM0B0  4
#define COM0B1  5
#define COM0A0  6
#define COM0A1  7
#define TCCR0B  __IO_MEM(0x45)
#define CS00    0
#define CS01    1
#define CS02    2
#define WGM02   3
#define FOC0B   6
#define FOC0A   7
#define TCNT0   __IO_MEM(0x46)
#define OCR0A   __IO_MEM(0x47)
#define OCR0B   __IO_MEM(0x48)

/* general purpose IO - page 25 */
#define GPIOR1  __IO_MEM(0x4A)
#define GPIOR2  __IO_MEM(0x4B)

/* SPI - page 167-169 */
#define SPCR    __IO_MEM(0x4C)
#define SPR0    0
#define SPR1    1
#define CPHA    2
#define CPOL    3
#define MSTR    4
#define DORD    5
#define SPE     6
#define SPIE    7
#define SPSR    __IO_MEM(0x4D)
#define SPI2X   0
#define WCOL    6
#define SPIF    7
#define SPDR    __IO_MEM(0x4E)

/* analog comparator - page 241 */
#define ACSR    __IO_MEM(0x50)
#define ACIS0   0
#define ACIS1   1
#define ACIC    2
#define ACIE    3
#define ACI     4
#define ACO     5
#define ACBG    6
#define ACD     7

/* power management - page 42 */
#define SMCR    __IO_MEM(0x53)
#define SE      0
#define SM0     1
#define SM1     2
#define SM2     3

/* MCU - page 52, 63, 86 */
#define MCUSR   __IO_MEM(0x54)
#define PORF    0
#define EXTRF   1
#define BORF    2
#define WDRF    3
#define MCUCR   __IO_MEM(0x55)
#define IVCE    0
#define IVSEL   1
#define PUD     4
#if defined(__AVR_mega48PA__) || defined(__AVR_mega88PA__) || defined(__AVR_mega168PA__) || defined(__AVR_mega328PA__)
#define BODSE   5
#define BODS    6
#endif

/* self-programming - page 282 */
#define SPMCSR  __IO_MEM(0x57)
#define SPMEN   0 /* the usual name for the SELFPRGEN bit as well */
#define SELFPRGEN   0
#define PGERS   1
#define PGWRT   2
#define BLBSET  3
#if !defined(__AVR_mega48__) && !defined(__AVR_mega48A__) && !defined(__AVR_mega48PA__)
#define RWWSRE  4
#define RWWSB   6
#endif
#define SPMIE   7

/* stack pointer / status register - page 10-12 */
#define SPL     __IO_MEM(0x5D)
#define SPH     __IO_MEM(0x5E)
#define SREG    __IO_MEM(0x5F)

/* watchdog - page 52 */
#define WDTCSR  __IO_MEM(0x60)
#define WDP0    0
#define WDP1    1
#define WDP2    2
#define WDE     3
#define WDCE    4
#define WDP3    5
#define WDIE    6
#define WDIF    7

/* clock prescale - page 36 */
#define CLKPR   __IO_MEM(0x61)
#define CLKPS0  0
#define CLKPS1  1
#define CLKPS2  2
#define CLKPS3  3
#define CLKPCE  7

/* power management - page 40 */
#define PRR     __IO_MEM(0x64)
#define PRADC   0
#define PRUSART0        1
#define PRSPI   2
#define PRTIM1  3
#define PRTIM0  5
#define PRTIM2  6
#define PRTWI   7

/* oscillator calibration - page 36 */
#define OSCCAL  __IO_MEM(0x66)

/* external / pin change interrupts - page 64-69 */
#define PCICR   __IO_MEM(0x68)
#define PCIE0   0
#define PCIE1   1
#define PCIE2   2
#define EICRA   __IO_MEM(0x69)
#define ISC00   0
#define ISC01   1
#define ISC10   2
#define ISC11   3
#define PCMSK0  __IO_MEM(0x6B)
#define PCINT0  0
#define PCINT1  1
#define PCINT2  2
#define PCINT3  3
#define PCINT4  4
#define PCINT5  5
#define PCINT6  6
#define PCINT7  7
#define PCMSK1  __IO_MEM(0x6C)
#define PCINT8  0
#define PCINT9  1
#define PCINT10 2
#define PCINT11 3
#define PCINT12 4
#define PCINT13 5
#define PCINT14 6
#define PCMSK2  __IO_MEM(0x6D)
#define PCINT16 0
#define PCINT17 1
#define PCINT18 2
#define PCINT19 3
#define PCINT20 4
#define PCINT21 5
#define PCINT22 6
#define PCINT23 7

/* timer interrupt masks - page 105, 134, 157 */
#define TIMSK0  __IO_MEM(0x6E)
#define TOIE0   0
#define OCIE0A  1
#define OCIE0B  2
#define TIMSK1  __IO_MEM(0x6F)
#define TOIE1   0
#define OCIE1A  1
#define OCIE1B  2
#define ICIE1   5
#define TIMSK2  __IO_MEM(0x70)
#define TOIE2   0
#define OCIE2A  1
#define OCIE2B  2

/* ADC / analog comparator - page 241-258 */
#define ADCL    __IO_MEM(0x78)
#define ADCH    __IO_MEM(0x79)
#define ADCSRA  __IO_MEM(0x7A)
#define ADPS0   0
#define ADPS1   1
#define ADPS2   2
#define ADIE    3
#define ADIF    4
#define ADATE   5
#define ADSC    6
#define ADEN    7
#define ADCSRB  __IO_MEM(0x7B)
#define ADTS0   0
#define ADTS1   1
#define ADTS2   2
#define ACME    6
#define ADMUX   __IO_MEM(0x7C)
#define MUX0    0
#define MUX1    1
#define MUX2    2
#define MUX3    3
#define ADLAR   5
#define REFS0   6
#define REFS1   7
#define DIDR0   __IO_MEM(0x7E)
#define ADC0D   0
#define ADC1D   1
#define ADC2D   2
#define ADC3D   3
#define ADC4D   4
#define ADC5D   5
#define DIDR1   __IO_MEM(0x7F)
#define AIN0D   0
#define AIN1D   1

/* timer 1 - page 129-134 */
#define TCCR1A  __IO_MEM(0x80)
#define WGM10   0
#define WGM11   1
#define COM1B0  4
#define COM1B1  5
#define COM1A0  6
#define COM1A1  7
#define TCCR1B  __IO_MEM(0x81)
#define CS10    0
#define CS11    1
#define CS12    2
#define WGM12   3
#define WGM13   4
#define ICES1   6
#define ICNC1   7
#define TCCR1C  __IO_MEM(0x82)
#define FOC1B   6
#define FOC1A   7
#define TCNT1L  __IO_MEM(0x84)
#define TCNT1H  __IO_MEM(0x85)
#define ICR1L   __IO_MEM(0x86)
#define ICR1H   __IO_MEM(0x87)
#define OCR1AL  __IO_MEM(0x88)
#define OCR1AH  __IO_MEM(0x89)
#define OCR1BL  __IO_MEM(0x8A)
#define OCR1BH  __IO_MEM(0x8B)

/* timer 2 - page 152-158 */
#define TCCR2A  __IO_MEM(0xB0)
#define WGM20   0
#define WGM21   1
#define COM2B0  4
#define COM2B1  5
#define COM2A0  6
#define COM2A1  7
#define TCCR2B  __IO_MEM(0xB1)
#define CS20    0
#define CS21    1
#define CS22    2
#define WGM22   3
#define FOC2B   6
#define FOC2A   7
#define TCNT2   __IO_MEM(0xB2)
#define OCR2A   __IO_MEM(0xB3)
#define OCR2B   __IO_MEM(0xB4)
#define ASSR    __IO_MEM(0xB6)
#define TCR2BUB 0
#define TCR2AUB 1
#define OCR2BUB 2
#define OCR2AUB 3
#define TCN2UB  4
#define AS2     5
#define EXCLK   6

/* TWI - page 235-238 */
#define TWBR    __IO_MEM(0xB8)
#define TWSR    __IO_MEM(0xB9)
#define TWPS0   0
#define TWPS1   1
#define TWS3    3
#define TWS4    4
#define TWS5    5
#define TWS6    6
#define TWS7    7
#define TWAR    __IO_MEM(0xBA)
#define TWGCE   0
#define TWA0    1
#define TWA1    2
#define TWA2    3
#define TWA3    4
#define TWA4    5
#define TWA5    6
#define TWA6    7
#define TWDR    __IO_MEM(0xBB)
#define TWCR    __IO_MEM(0xBC)
#define TWIE    0
#define TWEN    2
#define TWWC    3
#define TWSTO   4
#define TWSTA   5
#define TWEA    6
#define TWINT   7
#define TWAMR   __IO_MEM(0xBD)
#define TWAM0   1
#define TWAM1   2
#define TWAM2   3
#define TWAM3   4
#define TWAM4   5
#define TWAM5   6
#define TWAM6   7

/* USART - page 189-193, 205-207 */
#define UCSR0A  __IO_MEM(0xC0)
#define MPCM0   0
#define U2X0    1
#define UPE0    2
#define DOR0    3
#define FE0     4
#define UDRE0   5
#define TXC0    6
#define RXC0    7
#define UCSR0B  __IO_MEM(0xC1)
#define TXB80   0
#define RXB80   1
#define UCSZ02  2
#define TXEN0   3
#define RXEN0   4
#define UDRIE0  5
#define TXCIE0  6
#define RXCIE0  7
#define UCSR0C  __IO_MEM(0xC2)
#define UCPOL0  0
#define UCSZ00  1
#define UCPHA0  1
#define UCSZ01  2
#define UDORD0  2
#define USBS0   3
#define UPM00   4
#define UPM01   5
#define UMSEL00 6
#define UMSEL01 7
#define UBRR0L  __IO_MEM(0xC4)
#define UBRR0H  __IO_MEM(0xC5)
#define UDR0    __IO_MEM(0xC6)

/* 16bit registers */
#define EEAR    __IO_MEM16(0x41)
#define SP      __IO_MEM16(0x5D)
#define ADC     __IO_MEM16(0x78)
#define TCNT1   __IO_MEM16(0x84)
#define ICR1    __IO_MEM16(0x86)
#define OCR1A   __IO_MEM16(0x88)
#define OCR1B   __IO_MEM16(0x8A)
#define UBRR0   __IO_MEM16(0xC4)

/* interrupts */
#define __vector_INT0           __vector_1
#define __vector_INT1           __vector_2
#define __vector_PCINT0         __vector_3
#define __vector_PCINT1         __vector_4
#define __vector_PCINT2         __vector_5
#define __vector_WDT            __vector_6
#define __vector_TIMER2_COMPA   __vector_7
#define __vector_TIMER2_COMPB   __vector_8
#define __vector_TIMER2_OVF     __vector_9
#define __vector_TIMER1_CAPT    __vector_10
#define __vector_TIMER1_COMPA   __vector_11
#define __vector_TIMER1_COMPB   __vector_12
#define __vector_TIMER1_OVF     __vector_13
#define __vector_TIMER0_COMPA   __vector_14
#define __vector_TIMER0_COMPB   __vector_15
#define __vector_TIMER0_OVF     __vector_16
#define __vector_SPI_STC        __vector_17
#define __vector_USART0_RX      __vector_18
#define __vector_USART0_UDRE    __vector_19
#define __vector_USART0_TX      __vector_20
#define __vector_ADC            __vector_21
#define __vector_EE_READY       __vector_22
#define __vector_ANALOG_COMP    __vector_23
#define __vector_TWI            __vector_24
#define __vector_SPM_READY      __vector_25
