#ifndef __LIBAVR_EEPROM__
#define __LIBAVR_EEPROM__

#include <stdint.h>
#include <libavr/device.h>
#include <libavr/interrupt.h>

#define EEPROM __attribute__((section(".eeprom")))

typedef uint16_t eaddr_t;

#define eeprom_int8_t int8_t EEPROM
#define eeprom_int16_t int16_t EEPROM
#define eeprom_int32_t int32_t EEPROM
#define eeprom_int64_t int64_t EEPROM
#define eeprom_uint8_t uint8_t EEPROM
#define eeprom_uint16_t uint16_t EEPROM
#define eeprom_uint32_t uint32_t EEPROM
#define eeprom_uint64_t uint64_t EEPROM

#define eeprom_read(address) EEPROM_READ((eaddr_t) address)
#define eeprom_read16(address) EEPROM_READ16((eaddr_t) address)
#define eeprom_read32(address) EEPROM_READ32((eaddr_t) address)
#define eeprom_read64(address) EEPROM_READ64((eaddr_t) address)
#define eeprom_write(address, value) EEPROM_WRITE((eaddr_t) address, value)
#define eeprom_write16(address, value) EEPROM_WRITE16((eaddr_t) address, value)
#define eeprom_write32(address, value) EEPROM_WRITE32((eaddr_t) address, value)
#define eeprom_write64(address, value) EEPROM_WRITE64((eaddr_t) address, value)

#ifdef EEPE
#define EEWE EEPE
#define EEMWE EEMPE
#endif

static inline void eeprom_wait() {
    asm volatile ( "1: sbic %0, %1\n\t"
                   "rjmp 1b"
                   :
                   :
                   "i" (&EECR - 0x20),
                   "i" (EEWE) );
}

static inline uint8_t EEPROM_READ(eaddr_t address) {
    uint8_t data;
    asm volatile ( "1: sbic %1, %5\n\t"
                   "rjmp 1b\n\t"
                   "out %2, %A6\n\t"
                   "out %3, %B6\n\t"
                   "sbi %1, %7\n\t"
                   "in %0, %4"
                   :
                   "=d" (data)
                   :
                   "i" (&EECR - 0x20),
                   "i" (&EEARL - 0x20),
                   "i" (&EEARH - 0x20),
                   "i" (&EEDR - 0x20),
                   "i" (EEWE),
                   "d" (address),
                   "i" (EERE) );
    return data;
}

static inline void EEPROM_WRITE(eaddr_t address, uint8_t data) {
    asm volatile ( "1: sbic %0, %4\n\t"
                   "rjmp 1b\n\t"
                   "out %1, %A5\n\t"
                   "out %2, %B5\n\t"
                   "out %3, %6\n\t"
                   INTERRUPTS_OFF("r30")
                   "sbi %0, %7\n\t"
                   "sbi %0, %4\n\t"
                   INTERRUPTS_RESTORE("r30")
                   "clr r1\n\t"
                   :
                   :
                   "i" (&EECR - 0x20),
                   "i" (&EEARL - 0x20),
                   "i" (&EEARH - 0x20),
                   "i" (&EEDR - 0x20),
                   "i" (EEWE),
                   "d" (address),
                   "d" (data),
                   "i" (EEMWE)
                   :
                   "r30" );
}

#endif
