#ifndef __LIBAVR_FLASH_WRITE__
#define __LIBAVR_FLASH_WRITE__

#include <stdint.h>
#include <libavr/device.h>
#include <libavr/flash.h>

void flash_wait(void);
void flash_erase(faddr_t page);
void flash_write(faddr_t page, void *buf);

#ifndef SPMCR
#define SPMCR SPMCSR
#endif

static inline void flash_page_erase(faddr_t address) {
#if FLASHEND > 0xffff
    asm (
         "movw r30, %A3\n\t"
         "sts %1, %C3\n\t"
         "sts %0, %2\n\t"
         "spm\n\t"
         :
         : "i" (&SPMCR),
         "i" (&RAMPZ),
         "r" ((uint8_t) (1<<SPMEN | 1<<PGERS)),
         "r" (address)
         : "r30", "r31"
         );
#else
    asm (
         "movw r30, %2\n\t"
         "sts %0, %1\n\t"
         "spm\n\t"
         :
         : "i" (&SPMCR),
         "r" ((uint8_t) (1<<SPMEN | 1<<PGERS)),
         "r" (address)
         : "r30", "r31"
         );
#endif
}

static inline void flash_page_fill(faddr_t address, uint16_t data) {
#if FLASHEND > 0xffff
    asm (
        "movw  r0, %4\n\t"
        "movw r30, %A3\n\t"
        "sts %1, %C3\n\t"
        "sts %0, %2\n\t"
        "spm\n\t"
        "clr  r1\n\t"
        :
        : "i" (&SPMCR),
          "i" (&RAMPZ),
          "r" ((uint8_t) (1<<SPMEN)),
          "r" (address),
          "r" (data)
        : "r0", "r30", "r31"
        );
#else
    asm (
         "movw r30, %2\n\t"
         "sts %0, %1\n\t"
         "spm\n\t"
         :
         : "i" (&SPMCR),
         "r" ((uint8_t) (1<<SPMEN)),
         "r" (address)
         : "r30", "r31"
         );
#endif
}

static inline void flash_page_write(faddr_t address) {
#if FLASHEND > 0xffff
    asm (
         "movw r30, %A3\n\t"
         "sts %1, %C3\n\t"
         "sts %0, %2\n\t"
         "spm\n\t"
         :
         : "i" (&SPMCR),
         "i" (&RAMPZ),
         "r" ((uint8_t) (1<<SPMEN | 1<<PGWRT)),
         "r" (address)
         : "r30", "r31"
         );
#else
    asm (
        "movw r30, %2\n\t"
        "sts %0, %1\n\t"
        "spm\n\t"
        :
        : "i" (&SPMCR),
          "r" ((uint8_t) (1<<SPMEN | 1<<PGWRT)),
          "r" (address)
        : "r30", "r31"
         );
#endif
}

static inline void flash_rww_enable(void) {
    asm (
         "sts %0, %1\n\t"
         "spm\n\t"
         :
         : "i" (&SPMCR),
         "r" ((uint8_t) (1<<SPMEN | 1<<RWWSRE))
         );
}

#endif
