#ifndef __LIBAVR_UART__
#define __LIBAVR_UART__

#include <libavr/bits.h>
#include <libavr/baud.h>

#ifdef URSEL
#define BIT_URSEL 1<<URSEL
#else
#define BIT_URSEL 0
#endif

#define UART_INIT(PORT, BAUD, PARITY, BITS, STOP) \
    do { \
        UBRR##PORT##H = 0; \
        UBRR##PORT##L = BAUD; \
        UCSR##PORT##A = 1<<(U2X##PORT); /* enable 2x mode */ \
        UCSR##PORT##B = 1<<(RXEN##PORT) | 1<<(TXEN##PORT) /* enable both tx and rx */ \
            | UART_UCSRB_BITS(PORT, BITS); \
        UCSR##PORT##C = BIT_URSEL \
            | UART_UCSRC_PARITY(PORT, PARITY) \
            | UART_UCSRC_BITS(PORT, BITS) \
            | UART_UCSRC_STOP(PORT, STOP); \
    } while(0)

#define UART_UCSRC_PARITY(PORT, PARITY) \
    (  (PARITY == 'N') ? 0<<UPM##PORT##1 | 0<<UPM##PORT##0 \
     : (PARITY == 'E') ? 1<<UPM##PORT##1 | 0<<UPM##PORT##0 \
     : /* otherwise */   1<<UPM##PORT##1 | 1<<UPM##PORT##0 )

#define UART_UCSRB_BITS(PORT, BITS) \
    (  (BITS == 9) ?   1<<UCSZ##PORT##2 \
     : /* otherwise */ 0<<UCSZ##PORT##2 )
#define UART_UCSRC_BITS(PORT, BITS) \
    (  (BITS == 5) ?   0<<UCSZ##PORT##1 | 0<<UCSZ##PORT##0 \
     : (BITS == 6) ?   0<<UCSZ##PORT##1 | 1<<UCSZ##PORT##0 \
     : (BITS == 7) ?   1<<UCSZ##PORT##1 | 0<<UCSZ##PORT##0 \
     : /* otherwise */ 1<<UCSZ##PORT##1 | 1<<UCSZ##PORT##0 )

#define UART_UCSRC_STOP(PORT, STOP) \
    ((STOP == 2 ? 1 : 0) << USBS##PORT)

/* macros to deal with rs485 drivers or half-duplex buses */
#define UART_INIT_RS485(PORT, BAUD, PARITY, BITS, STOP) \
    do { \
        UART_INIT(PORT, BAUD, PARITY, BITS, STOP); \
        INIT_UART##PORT##_ENABLE_TX; \
        INIT_UART##PORT##_ENABLE_RX; \
        UART_ENABLE_RX(PORT); \
    } while(0)

#define UART_ENABLE_TX(PORT) \
    do { \
        CBI_UART##PORT##_ENABLE_RX; \
        SBI_UART##PORT##_ENABLE_TX; \
        cbi(UCSR##PORT##B, RXEN##PORT); \
        sbi(UCSR##PORT##B, TXEN##PORT); \
    } while(0)

#define UART_ENABLE_RX(PORT) \
    do { \
        CBI_UART##PORT##_ENABLE_TX; \
        SBI_UART##PORT##_ENABLE_RX; \
        cbi(UCSR##PORT##B, TXEN##PORT); \
        sbi(UCSR##PORT##B, RXEN##PORT); \
    } while(0)


/* UART0 helper macros */
#ifdef BIT_UART0_ENABLE_TX
#define INIT_UART0_ENABLE_TX SBI(DDR, UART0_ENABLE_TX)
#define SBI_UART0_ENABLE_TX SBI(PORT, UART0_ENABLE_TX)
#define CBI_UART0_ENABLE_TX CBI(PORT, UART0_ENABLE_TX)
#else
#define INIT_UART0_ENABLE_TX
#define SBI_UART0_ENABLE_TX
#define CBI_UART0_ENABLE_TX
#endif

#ifdef BIT_UART0_ENABLE_RX
#define INIT_UART0_ENABLE_RX SBI(DDR, UART0_ENABLE_RX)
#define SBI_UART0_ENABLE_RX SBI(PORT, UART0_ENABLE_RX)
#define CBI_UART0_ENABLE_RX CBI(PORT, UART0_ENABLE_RX)
#else
#define INIT_UART0_ENABLE_RX
#define SBI_UART0_ENABLE_RX
#define CBI_UART0_ENABLE_RX
#endif

/* UART1 helper macros */
#ifdef BIT_UART1_ENABLE_TX
#define INIT_UART1_ENABLE_TX SBI(DDR, UART1_ENABLE_TX)
#define SBI_UART1_ENABLE_TX SBI(PORT, UART1_ENABLE_TX)
#define CBI_UART1_ENABLE_TX CBI(PORT, UART1_ENABLE_TX)
#else
#define INIT_UART1_ENABLE_TX
#define SBI_UART1_ENABLE_TX
#define CBI_UART1_ENABLE_TX
#endif

#ifdef BIT_UART1_ENABLE_RX
#define INIT_UART1_ENABLE_RX SBI(DDR, UART1_ENABLE_RX)
#define SBI_UART1_ENABLE_RX SBI(PORT, UART1_ENABLE_RX)
#define CBI_UART1_ENABLE_RX CBI(PORT, UART1_ENABLE_RX)
#else
#define INIT_UART1_ENABLE_RX
#define SBI_UART1_ENABLE_RX
#define CBI_UART1_ENABLE_RX
#endif

#endif
