#ifndef __LIBAVR_WATCHDOG__
#define __LIBAVR_WATCHDOG__

#include <libavr/interrupt.h>

#define watchdog_reset()       wdr()
#define wdr()                  asm volatile ( "wdr" )

#ifdef WDTCR
#define __WATCHDOG_REG WDTCR
#else
#define __WATCHDOG_REG WDTCSR
#endif

#ifdef MCUSR
#define __WATCHDOG_MCUSR MCUSR
#else
#define __WATCHDOG_MCUSR MCUCSR
#endif

#define watchdog_enable(timeout) \
    asm volatile ( \
        INTERRUPTS_OFF("r30") \
        "wdr\n\t" \
        "lds    r31, %0\n\t" \
        "ori    r31, %1\n\t" \
        "sts    %0, r31\n\t" \
        "ldi    r31, %2\n\t" \
        "sts    %0, r31\n\t" \
        INTERRUPTS_RESTORE("r30") \
        : \
        : \
        "i" (&__WATCHDOG_REG), \
        "i" ((1 << WDCE) | (1 << WDE)), \
        "i" ((1 << WDE) | timeout) \
        : \
        "r30", \
        "r31" )


#define watchdog_disable() \
    asm volatile ( \
        INTERRUPTS_OFF("r30") \
        "wdr\n\t" \
        "lds    r31, %0\n\t" \
        "andi   r31, %1\n\t" \
        "sts    %0, r31\n\t" \
        "lds    r31, %2\n\t" \
        "ori    r31, %3\n\t" \
        "sts    %2, r31\n\t" \
        "clr    r31\n\t" \
        "sts    %2, r31\n\t" \
        INTERRUPTS_RESTORE("r30") \
        : \
        : \
        "i" (&__WATCHDOG_MCUSR), \
        "i" (~(1 << WDRF)), \
        "i" (&__WATCHDOG_REG), \
        "i" ((1 << WDCE) | (1 << WDE)) \
        : \
        "r30", \
        "r31" )

#define WDT_16MS        0
#define WDT_32MS        (1 << WDP0)
#define WDT_64MS        (1 << WDP1)
#define WDT_125MS       ((1 << WDP1) | (1 << WDP0))
#define WDT_250MS       (1 << WDP2)
#define WDT_500MS       ((1 << WDP2) | (1 << WDP0))
#define WDT_1S          ((1 << WDP2) | (1 << WDP1))
#define WDT_2S          ((1 << WDP2) | (1 << WDP1) | (1 << WDP0))
#ifdef WDP3
#define WDT_4S          (1 << WDP3)
#define WDT_8S          ((1 << WDP3) | (1 << WDP0))
#endif

#endif
